#!/bin/bash -e
#
# $Id: build-updates.sh 18981 2022-10-02 00:56:46Z NiLuJe $
#

HACKNAME="linkfonts"
HACKDIR="Fonts"
PKGNAME="${HACKNAME}"
PKGVER="0.14.N"

# Setup KindleTool packaging metadata flags to avoid cluttering the invocations
PKGREV="$(svnversion -c .. | awk '{print $NF}' FS=':' | tr -d 'P')"
KT_PM_FLAGS=( "-xPackageName=${HACKDIR}" "-xPackageVersion=${PKGVER}-r${PKGREV}" "-xPackageAuthor=NiLuJe" "-xPackageMaintainer=NiLuJe" "-X" )

# We need kindletool (https://github.com/NiLuJe/KindleTool) in $PATH
if (( $(kindletool version | wc -l) == 1 )) ; then
	HAS_KINDLETOOL="true"
fi

if [[ "${HAS_KINDLETOOL}" != "true" ]] ; then
	echo "You need KindleTool (https://github.com/NiLuJe/KindleTool) to build this package."
	exit 1
fi

# We also need GNU tar
if [[ "$(uname -s)" == "Darwin" ]] ; then
	TAR_BIN="gtar"
else
	TAR_BIN="tar"
fi
if ! ${TAR_BIN} --version | grep -q "GNU tar" ; then
	echo "You need GNU tar to build this package."
	exit 1
fi

## Install

# Go away if we don't have the PW2 tree checked out for the A9 binaries...
if [[ ! -d "../../../PW2_Hacks" ]] ; then
	echo "Skipping Fonts build, we're missing the A9 binaries (from the PW2_Hacks tree)"
	exit 1
fi

## Go away if we don't have the Fonts tree for the legacy version checked out...
if [[ ! -d "../../../Hacks/Fonts" ]] ; then
	echo "Skipping Fonts build, we're missing the KUAL extension (from the Hacks tree)"
	exit 1
fi

# Pickup our common stuff... We leave it in our staging wd so it ends up in the source package.
if [[ ! -d "../../Common" ]] ; then
	echo "The tree isn't checked out in full, missing the Common directory..."
	exit 1
fi
# LibOTAUtils 5
ln -f ../../Common/lib/libotautils5 ./libotautils5
# XZ Utils
ln -f ../../Common/bin/xzdec ./xzdec
# LibKH 5
for common_lib in libkh5 ; do
	ln -f ../../Common/lib/${common_lib} ../src/${HACKNAME}/bin/${common_lib}
done
# USB Watchdog
for common_bin in usb-watchdog usb-watchdog-helper ; do
	ln -f ../../Common/bin/${common_bin} ../src/${HACKNAME}/bin/${common_bin}
done
# FBInk
ln -f ../../Common/bin/fbink ../src/${HACKNAME}/bin/fbink

# Make sure we bundle our KUAL extension...
cp -avf ../../../Hacks/Fonts/src/extensions/linkfonts ../src/extensions


# Archive custom directory
export XZ_DEFAULTS="-T 0"
${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs -cvJf ${HACKNAME}.tar.xz ../src/${HACKNAME} ../src/extensions

# Copy the script to our working directory, to avoid storing crappy paths in the update package
ln -f ../src/install.sh ./
ln -f ../src/${HACKNAME}.conf ./
ln -f ../src/75-linkfonts-altfonts.conf ./

# Build the install package. We only support FW >= 5.3.1, so switch to an OTA update with a version check.
kindletool create ota2 "${KT_PM_FLAGS[@]}" -d touch -d paperwhite -b FC04 -s 1795370024 libotautils5 install.sh ${HACKNAME}.tar.xz xzdec ${HACKNAME}.conf 75-linkfonts-altfonts.conf Update_${PKGNAME}_${PKGVER}_install_touch_pw.bin

# Remove the Touch & PaperWhite archive
rm -f ./${HACKNAME}.tar.xz

# Build the PaperWhite 2 archive...
${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs -cvf ${HACKNAME}.tar ../src/${HACKNAME} ../src/extensions
# Delete A8 binaries
KINDLE_MODEL_BINARIES="bin/fc-scan bin/fc-list bin/shlock lib/autohint/libfreetype.so lib/bci/libfreetype.so lib/light/libfreetype.so lib/spr/libfreetype.so lib/libharfbuzz.so.0 lib/libexpat.so.1 lib/libfontconfig.so.1 lib/libz.so.1 lib/libfreetype.so.6 lib/libuuid.so.1"
for my_bin in ${KINDLE_MODEL_BINARIES} ; do
	${TAR_BIN} --delete -vf ${HACKNAME}.tar src/${HACKNAME}/${my_bin}
done
# Append A9 binaries
for my_bin in ${KINDLE_MODEL_BINARIES} ; do
	${TAR_BIN} --hard-dereference --owner root --group root --transform "s,^PW2_Hacks/${HACKDIR}/,,S" --show-transformed-names -rvf ${HACKNAME}.tar ../../../PW2_Hacks/${HACKDIR}/src/${HACKNAME}/${my_bin}
done
# Do the same for FBInk
${TAR_BIN} --delete -vf ${HACKNAME}.tar src/${HACKNAME}/bin/fbink
${TAR_BIN} --hard-dereference --owner root --group root --transform "s,^PW2_Hacks/Common/,src/${HACKNAME}/,S" --show-transformed-names -rvf ${HACKNAME}.tar ../../../PW2_Hacks/Common/bin/fbink
# xz it...
xz ${HACKNAME}.tar

# Speaking of, we need our own xzdec binary, too!
ln -f ../../../PW2_Hacks/Common/bin/xzdec ./xzdec

# Build the install package (>= Wario)
kindletool create ota2 "${KT_PM_FLAGS[@]}" -d paperwhite2 -d basic -d voyage -d paperwhite3 -d oasis -d basic2 -d oasis2 -d paperwhite4 -d basic3 -d oasis3 -d paperwhite5 -d scribe libotautils5 install.sh ${HACKNAME}.tar.xz xzdec ${HACKNAME}.conf 75-linkfonts-altfonts.conf Update_${PKGNAME}_${PKGVER}_install_pw2_and_up.bin

## Uninstall
# Copy the script to our working directory, to avoid storing crappy paths in the update package
ln -f ../src/uninstall.sh ./

# Build the uninstall package
kindletool create ota2 "${KT_PM_FLAGS[@]}" -d kindle5 libotautils5 uninstall.sh Update_${PKGNAME}_${PKGVER}_uninstall.bin

## Cleanup
# Remove package specific temp stuff
rm -f ./install.sh ./uninstall.sh ./${HACKNAME}.tar.xz ./xzdec ./${HACKNAME}.conf ./75-linkfonts-altfonts.conf

# Move our updates
mv -f *.bin ../
