#!/bin/sh
#
# KUAL Font Overrides actions helper script
#
# $Id: linkfonts.sh 18979 2022-10-02 00:48:56Z NiLuJe $
#
##

# Get hackname from the script's path (NOTE: Will only work for scripts called from /mnt/us/extensions/${KH_HACKNAME})
#KH_HACKNAME="${PWD##/mnt/us/extensions/}"
# We're always working with linkfonts, hardcode it.
KH_HACKNAME="linkfonts"

# Try to pull our custom helper lib
libkh_fail="false"
# Handle both the K5 & legacy helper, so I don't have to maintain the exact same thing in two different places :P
for my_libkh in libkh5 libkh ; do
	_KH_FUNCS="/mnt/us/${KH_HACKNAME}/bin/${my_libkh}"
	if [ -f ${_KH_FUNCS} ] ; then
		. ${_KH_FUNCS}
		# Got it, go away!
		libkh_fail="false"
		break
	else
		libkh_fail="true"
	fi
done

if [ "${libkh_fail}" == "true" ] ; then
	# Pull default helper functions for logging
	_FUNCTIONS=/etc/rc.d/functions
	[ -f ${_FUNCTIONS} ] && . ${_FUNCTIONS}
	# We couldn't get our custom lib, abort
	msg "couldn't source libkh5 nor libkh from '${KH_HACKNAME}'" W
	exit 0
fi

# We don't especially need full privileges, but warn about it anyway :)
if [ "$(id -u)" -ne 0 ] ; then
	kh_msg "unprivileged user" W q
fi

## Enable a specific custom Fontconfig config shard
# Arg 1 is exact config name
##
enable_fontconfig_shard()
{
	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to enable_fontconfig_shard ($# while we need at least 1)" W v "missing config file name"
	fi

	fc_shard_in_pool="${LINKFONTS_CONFDIR}/conf.avail/${1}.conf"
	fc_shard_enabled="${LINKFONTS_CONFDIR}/conf.d/${1}.conf"

	if [ ! -f "${fc_shard_in_pool}" ] ; then
		kh_msg "Config shard ${1} not available" W v
		return 1
	fi

	cp -f "${fc_shard_in_pool}" "${fc_shard_enabled}"
}

## Remove a specific custom Fontconfig config shard
# Arg 1 is exact config name
##
disable_fontconfig_shard()
{
	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to disable_fontconfig_shard ($# while we need at least 1)" W v "missing config file name"
		return 1
	fi

	fc_shard_enabled="${LINKFONTS_CONFDIR}/conf.d/${1}.conf"

	rm -f "${fc_shard_enabled}"
}

## Check if we're a K5/PW/KV/PW3/KOA/KT3/KOA2/PW4/KT4/KOA3/PW5
check_is_touch_device()
{
	[ "${IS_K5}" == "true" ] && return 0
	[ "${IS_TOUCH}" == "true" ] && return 0
	[ "${IS_PW}" == "true" ] && return 0
	[ "${IS_PW2}" == "true" ] && return 0
	[ "${IS_KV}" == "true" ] && return 0
	[ "${IS_KT2}" == "true" ] && return 0
	[ "${IS_PW3}" == "true" ] && return 0
	[ "${IS_KOA}" == "true" ] && return 0
	[ "${IS_KT3}" == "true" ] && return 0
	[ "${IS_KOA2}" == "true" ] && return 0
	[ "${IS_PW4}" == "true" ] && return 0
	[ "${IS_KT4}" == "true" ] && return 0
	[ "${IS_KOA3}" == "true" ] && return 0
	[ "${IS_PW5}" == "true" ] && return 0
	[ "${IS_KS}" == "true" ] && return 0

	# We're not!
	return 1
}

## Rebuild the menu
rebuild_menu()
{
	# Go away if we don't have Python...
	if [ ! -f "/mnt/us/python/bin/python2.7" ] ; then
		kh_msg "can't do that: python is not installed" W v "python is not installed"
		return 1
	fi

	# Need to shift a bit to have consistent positional params
	shift

	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to rebuild_menu ($# while we need at least 1)" W v "missing menu type"
		return 1
	fi

	menu_type="${1}"

	# We rely on a Python script, because it's a bit more elegantly done in Python, but more importantly, because busybox's ash is slow as hell.
	/mnt/us/python/bin/python2.7 "${PWD}/bin/rebuild_menu.py" "${menu_type}"
}

## Reset default serif family
default_serif()
{
	disable_fontconfig_shard "98-override-serif"
	kh_msg "Restore default serif family" I a
}

## Reset default sans family
default_sans()
{
	disable_fontconfig_shard "98-override-sans"
	kh_msg "Restore default sans family" I a
}

## Reset default mono family
default_mono()
{
	disable_fontconfig_shard "98-override-mono"
	kh_msg "Restore default mono family" I a
}

## Reset default condensed family
default_condensed()
{
	disable_fontconfig_shard "98-override-condensed"
	kh_msg "Restore default condensed family" I a
}

## Reset default fallback font
default_fallback()
{
	disable_fontconfig_shard "98-override-fallback"
	kh_msg "Restore default fallback font" I a
}

## Build a specific family override
# Arg 1 is the family to override
# Arg 2 is the family to use
##
override_family()
{
	# Need to shift a bit to have consistent positional params
	shift

	if [ $# -lt 2 ] ; then
		kh_msg "not enough arguments passed to override_family ($# while we need at least 2)" W v "missing family names"
		return 1
	fi

	fam_to_override="${1}"
	fam_to_use="${2}"

	fc_shard_name="98-override-${fam_to_override}"

	# Use the right alias for sans & mono...
	case "${fam_to_override}" in
		"sans" )
			fam_to_override="sans-serif"
		;;
		"mono" )
			fam_to_override="monospace"
		;;
	esac

	# The serif family is a bit trickier, because Java uses the "Caecilia Regular" (or "Bookerly" since FW 5.6.5) alias for the default logical serif font...
	if [ "${fam_to_override}" == "serif" ] ; then
		# FW >= 5.6.5 switched to Bookerly, handle that properly...
		if [ -f "/usr/java/lib/fonts/Bookerly-Regular.ttf" ] ; then
			sed -e "s/%TO_USE%/${fam_to_use}/g" "${LINKFONTS_CONFDIR}/fc-override-serif-bookerly.tpl" > "${LINKFONTS_CONFDIR}/conf.d/${fc_shard_name}.conf"
		else
			sed -e "s/%TO_USE%/${fam_to_use}/g" "${LINKFONTS_CONFDIR}/fc-override-serif.tpl" > "${LINKFONTS_CONFDIR}/conf.d/${fc_shard_name}.conf"
		fi
	# So does the sans family, with "Helvetica Neue LT"...
	elif [ "${fam_to_override}" == "sans-serif" ] ; then
		# FW >= 5.7.2 switched to Amazon Ember, handle that properly...
		if [ -f "/usr/java/lib/fonts/Amazon-Ember-Regular.ttf" ] ; then
			sed -e "s/%TO_USE%/${fam_to_use}/g" "${LINKFONTS_CONFDIR}/fc-override-sans-ember.tpl" > "${LINKFONTS_CONFDIR}/conf.d/${fc_shard_name}.conf"
		else
			sed -e "s/%TO_USE%/${fam_to_use}/g" "${LINKFONTS_CONFDIR}/fc-override-sans.tpl" > "${LINKFONTS_CONFDIR}/conf.d/${fc_shard_name}.conf"
		fi
	elif [ "${fam_to_override}" == "fallback" ] ; then
		# As for the fallback font, it's just Code2000, but it happens to be at the bottom of nearly every alias, so we have to take a different approach...
		sed -e "s/%TO_USE%/${fam_to_use}/g" "${LINKFONTS_CONFDIR}/fc-override-fallback.tpl" > "${LINKFONTS_CONFDIR}/conf.d/${fc_shard_name}.conf"
	else
		sed -e "s/%TO_OVERRIDE%/${fam_to_override}/g;s/%TO_USE%/${fam_to_use}/g" "${LINKFONTS_CONFDIR}/fc-override.tpl" > "${LINKFONTS_CONFDIR}/conf.d/${fc_shard_name}.conf"
	fi

	kh_msg "Using ${fam_to_use} as the default ${fam_to_override} font" I a "${fam_to_override} => ${fam_to_use}"
}

## Show the current default font for a specific generic family
# Arg 1 is the family
##
show_current_override()
{
	# Need to shift a bit to have consistent positional params
	shift

	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to show_current_override ($# while we need at least 1)" W v "missing family names"
		return 1
	fi

	fam_to_show="${1}"

	# Use the proper font name when asking for the fallback font....
	if [ "${fam_to_show}" == "fallback" ] ; then
		fam_to_show="Code2000"
	fi
	# Query Helvetica directly when asking for the sans font, because the fallback override affects the sans-serif alias...
	if [ "${fam_to_show}" == "sans" ] ; then
		fam_to_show="Helvetica Neue LT"
	fi

	default_font="$(fc-match -f "%{family[0]}" "${fam_to_show}")"

	kh_msg "Current default font for ${fam_to_show} is ${default_font}" I v "${fam_to_show} => ${default_font}"
}


## Main
case "${1}" in
	"rebuild_menu" )
		${1} "$@"
	;;
	"default_serif" )
		${1}
	;;
	"default_sans" )
		${1}
	;;
	"default_mono" )
		${1}
	;;
	"default_condensed" )
		${1}
	;;
	"default_fallback" )
		${1}
	;;
	"override_family" )
		${1} "$@"
	;;
	"show_current_override" )
		${1} "$@"
	;;
	* )
		kh_msg "invalid action (${1})" W v "invalid action"
	;;
esac
