#!/bin/sh
#
# KUAL Fonts actions helper script
#
# $Id: linkfonts.sh 18979 2022-10-02 00:48:56Z NiLuJe $
#
##

# Get hackname from the script's path (NOTE: Will only work for scripts called from /mnt/us/extensions/${KH_HACKNAME})
KH_HACKNAME="${PWD##/mnt/us/extensions/}"

# Try to pull our custom helper lib
libkh_fail="false"
# Handle both the K5 & legacy helper, so I don't have to maintain the exact same thing in two different places :P
for my_libkh in libkh5 libkh ; do
	_KH_FUNCS="/mnt/us/${KH_HACKNAME}/bin/${my_libkh}"
	if [ -f ${_KH_FUNCS} ] ; then
		. ${_KH_FUNCS}
		# Got it, go away!
		libkh_fail="false"
		break
	else
		libkh_fail="true"
	fi
done

if [ "${libkh_fail}" == "true" ] ; then
	# Pull default helper functions for logging
	_FUNCTIONS=/etc/rc.d/functions
	[ -f ${_FUNCTIONS} ] && . ${_FUNCTIONS}
	# We couldn't get our custom lib, abort
	msg "couldn't source libkh5 nor libkh from '${KH_HACKNAME}'" W
	exit 0
fi

# We need the proper privileges...
if [ "$(id -u)" -ne 0 ] ; then
	kh_msg "unprivileged user, aborting" E v
	exit 1
fi

## Enable a specific trigger file in the hack's basedir
# Arg 1 is exact config trigger file name
##
enable_hack_trigger_file()
{
	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to enable_hack_trigger_file ($# while we need at least 1)" W v "missing trigger file name"
	fi

	kh_trigger_file="${KH_HACK_BASEDIR}/${1}"

	touch "${kh_trigger_file}"
}

## Remove a specific trigger file in the hack's basedir
# Arg 1 is exact config trigger file name
##
disable_hack_trigger_file()
{
	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to disable_hack_trigger_file ($# while we need at least 1)" W v "missing trigger file name"
		return 1
	fi

	kh_trigger_file="${KH_HACK_BASEDIR}/${1}"

	rm -f "${kh_trigger_file}"
}

## Enable a specific custom Fontconfig config shard
# Arg 1 is exact config name
##
enable_fontconfig_shard()
{
	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to enable_fontconfig_shard ($# while we need at least 1)" W v "missing config file name"
	fi

	fc_shard_in_pool="${LINKFONTS_CONFDIR}/conf.avail/${1}.conf"
	fc_shard_enabled="${LINKFONTS_CONFDIR}/conf.d/${1}.conf"

	if [ ! -f "${fc_shard_in_pool}" ] ; then
		kh_msg "Config shard ${1} not available" W v
		return 1
	fi

	cp -f "${fc_shard_in_pool}" "${fc_shard_enabled}"
}

## Remove a specific custom Fontconfig config shard
# Arg 1 is exact config name
##
disable_fontconfig_shard()
{
	if [ $# -lt 1 ] ; then
		kh_msg "not enough arguments passed to disable_fontconfig_shard ($# while we need at least 1)" W v "missing config file name"
		return 1
	fi

	fc_shard_enabled="${LINKFONTS_CONFDIR}/conf.d/${1}.conf"

	rm -f "${fc_shard_enabled}"
}

## Check if we're a K3
check_is_k3_device()
{
	[ "${IS_K3}" == "true" ] && return 0
	# It's the fonts hack, so this is redundant, but we mean the K4, too ;)
	[ "${IS_K4}" == "true" ] && return 0

	# We're not!
	return 1
}

## Check if we're a K5+
check_is_touch_device()
{
	[ "${IS_K5}" == "true" ] && return 0
	[ "${IS_TOUCH}" == "true" ] && return 0
	[ "${IS_PW}" == "true" ] && return 0
	[ "${IS_PW2}" == "true" ] && return 0
	[ "${IS_KV}" == "true" ] && return 0
	[ "${IS_KT2}" == "true" ] && return 0
	[ "${IS_PW3}" == "true" ] && return 0
	[ "${IS_KOA}" == "true" ] && return 0
	[ "${IS_KT3}" == "true" ] && return 0
	[ "${IS_KOA2}" == "true" ] && return 0
	[ "${IS_PW4}" == "true" ] && return 0
	[ "${IS_KT4}" == "true" ] && return 0
	[ "${IS_KOA3}" == "true" ] && return 0
	[ "${IS_PW5}" == "true" ] && return 0
	[ "${IS_KS}" == "true" ] && return 0

	# We're not!
	return 1
}

## Enable at boot
enable_auto()
{
	enable_hack_trigger_file "auto"

	# Make sure our potentially disabled custom FC shards are re-enabled on FW 5.x...
	if check_is_touch_device ; then
		if [ -d "${LINKFONTS_CONFDIR}/conf.disabled" ] ; then
			# FIXME: Not terribly picky about what happens if the user messed with the folders manually in the meantime...
			mv -f "${LINKFONTS_CONFDIR}/conf.disabled" "${LINKFONTS_CONFDIR}/conf.d"
		fi
	fi

	# FIXME: Workaround broken? custom status message by using eips ourselves. Kill this once it works properly.
	kh_msg "Enable the Fonts Hack" I a
}

## Disable at boot
disable_auto()
{
	disable_hack_trigger_file "auto"

	# Disable our custom FC shards on FW 5.x...
	if check_is_touch_device ; then
		if [ -d "${LINKFONTS_CONFDIR}/conf.d" ] ; then
			mv -f "${LINKFONTS_CONFDIR}/conf.d" "${LINKFONTS_CONFDIR}/conf.disabled"

			# Hint that a restart would be a good idea...
			kh_msg "custom fontconfig shards disabled, you will need to restart your device at one point" I v "done. restart framework/X, then the device"
			sleep 2
		fi
	fi

	kh_msg "Disable the Fonts Hack" I a
}

## Enable verbose mode
enable_verbose()
{
	enable_hack_trigger_file "verbose"
	kh_msg "Make the Fonts Hack verbose" I a
}

## Disable verbose mode
disable_verbose()
{
	disable_hack_trigger_file "verbose"
	kh_msg "Make the Fonts Hack quiet" I a
}

## Enable complete uninstall flag
enable_uninstall()
{
	enable_hack_trigger_file "uninstall"
	kh_msg "Flag Fonts for complete uninstall" I a
}

## Disable complete uninstall flag
disable_uninstall()
{
	disable_hack_trigger_file "uninstall"
	kh_msg "Restore default Fonts uninstall behavior" I a
}

## Enable autoreboot feature
enable_autoreboot()
{
	enable_hack_trigger_file "autoreboot"
	kh_msg "Enable the autoreboot feature" I a
}

## Disable verbose mode
disable_autoreboot()
{
	# Since we take precedence over the ScreenSvers watchodg, we assume no means no, si kill it for SS too
	if [ -f "/mnt/us/linkss/autoreboot" ] ; then
		rm -f "/mnt/us/linkss/autoreboot"
	fi
	disable_hack_trigger_file "autoreboot"
	kh_msg "Disable the autoreboot feature" I a
}

## Trigger autoreboot feature
trigger_autoreboot()
{
	# Sanity check, check that there is at least one watchdog running (here, ours)
	if [ -f "${WATCHDOG_PID}" ] ; then
		enable_hack_trigger_file "reboot"
		kh_msg "Trigger a framework restart the next time device is unplugged (${KH_HACKNAME})" I a "trigger a framework restart at unplug (via F)"
		return 0
	fi

	# If we're here, then there's no watchdog running, do nothing, and print a warning
	kh_msg "Cannot do that without a watchdog running! Enable the autoreboot feature first?" W v "autoreboot feature not enabled"
	return 1
}

## Abort scheduled autoreboot
abort_autoreboot()
{
	# Handle both hacks, again
	if [ -f "/mnt/us/linkss/reboot" ] ; then
		rm -f "/mnt/us/linkss/reboot"
	fi
	if [ -f "${KH_HACK_BASEDIR}/reboot" ] ; then
		disable_hack_trigger_file "reboot"
	fi
	kh_msg "Abort the framework restart scheduled the next time the device is unplugged" I a "framework restart unscheduled"
}

## Restart framework...
framework_restart()
{
	# Sleep a while to let KUAL die
	kh_msg "Hush, little baby . . ." I a
	sleep 5

	# If the fonts hack is enabled, do the same tasks an autoreboot would...
	if [ -f "${LINKFONTS_BASEDIR}/auto" ] ; then
		${FONTCONFIG_HELPER} kual
	fi

	# Sync, just to be safe
	sync

	# Restart!
	kh_msg "Framework restart . . ." I v

	# FW >= 5.x
	if check_is_touch_device ; then
		# Show the splash screen...
		. /etc/upstart/splash
		splash_cleanup
		splash_init || kh_msg "cannot init splash module" E a
		splash_progress 0

		restart webreader
		# If we have it, restart the kfxreader, too
		[ -f "/etc/upstart/kfxreader.conf" ] && restart kfxreader

		# Survive the mass killing so we can properly update the progress bar...
		trap "" SIGTERM

		restart framework

		# And, apparently, we have to do everything ourselves...
		# That, or restart lab126_gui instead, but, hey, this works fine, and is probably a tiny but faster :).
		for splash_bar in $(seq 1 25) ; do
			splash_progress $((splash_bar * 4))
			sleep 1
		done
	else
		[ -x /etc/init.d/webreaderd ] && /etc/init.d/webreaderd restart
		[ -x /etc/init.d/browserd ] && /etc/init.d/browserd restart
		/etc/init.d/framework restart
	fi
}

## Restart webreader...
webreader_restart()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# Sleep a while to let KUAL die
	kh_msg "Hush, little baby . . ." I a
	sleep 4

	# If the fonts hack is enabled, do the same tasks an autoreboot would...
	if [ -f "${LINKFONTS_BASEDIR}/auto" ] ; then
		${FONTCONFIG_HELPER} kual
	fi

	# Sync, just to be safe
	sync

	# Restart!
	restart webreader
	[ -f "/etc/upstart/kfxreader.conf" ] && restart kfxreader

	# Feedback...
	kh_msg "WebReader restarted" I v
}

## Restart X...
xorg_restart()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# Sleep a while to let KUAL die
	kh_msg "Hush, little baby . . ." I a
	sleep 5

	# If the fonts hack is enabled, do the same tasks an autoreboot would...
	if [ -f "${LINKFONTS_BASEDIR}/auto" ] ; then
		${FONTCONFIG_HELPER} kual
	fi

	# Sync, just to be safe
	sync

	# Restart!
	kh_msg "Xorg restart . . ." I v

	# Survive the mass killing, even if we don't really need it for the progress bar, because lab126_gui will take care of that for us ;).
	# Actually catching SIGTERM to run a custom callback messed with upstart in insidious ways (we never hit the stopped state,
	# or at least the corresponding event wasn't fired, even though x & co were shut down properly, so linkfonts, with it's "stop on stopped x" stanza
	# wasn't actually stopped...), and since we don't actually need to do anything besides not die, just ignoring SIGTERM (trap "" sigspec) does the trick ;).
	# NOTE: Might be related to how the signal handler is propagated (or not) to child processes depending on how the block of code is quoted (or not),
	# or prepended with a colon... One of these days, check busybox ash's sources to see if/how they follow bash on this...
	trap "" SIGTERM

	# And... GO! Upstart will handle the domino effect for us (it downs lab126_gui, which downs the framework & friends)
	stop x
	# We need the stop and start because restart doesn't go through the 'stopped' phase, and we rely on it for our stop on stanza...
	# Cf. http://upstart.ubuntu.com/cookbook/#restart restart != stop && start.
	sync
	# This way, we make sure linkfonts & linkss will go down, and then go up again :)
	start x
}

## Switch back to Amazon's FT lib
ft_amazon()
{
	# It's the default, so just clear every other mode ;)
	for ft_lib in bci light autohint spr ; do
		[ -f "${KH_HACK_BASEDIR}/${ft_lib}" ] && disable_hack_trigger_file "${ft_lib}"
		check_is_touch_device && disable_fontconfig_shard "95-force-${ft_lib}"
	done
	kh_msg "Use the default FreeType renderer" I a
}

## Switch to the BCI FT lib
ft_bci()
{
	# Keep things tidy, to avoid getting bit on the ass by the priority rules
	for ft_lib in light autohint spr ; do
		[ -f "${KH_HACK_BASEDIR}/${ft_lib}" ] && disable_hack_trigger_file "${ft_lib}"
		check_is_touch_device && disable_fontconfig_shard "95-force-${ft_lib}"
	done
	# Then enable it
	enable_hack_trigger_file "bci"
	check_is_touch_device && enable_fontconfig_shard "95-force-bci"
	kh_msg "Use the custom BCI FreeType renderer" I a
}

## Switch to the Light FT lib
ft_light()
{
	# Keep things tidy, to avoid getting bit on the ass by the priority rules
	for ft_lib in bci autohint spr ; do
		[ -f "${KH_HACK_BASEDIR}/${ft_lib}" ] && disable_hack_trigger_file "${ft_lib}"
		check_is_touch_device && disable_fontconfig_shard "95-force-${ft_lib}"
	done
	# Then enable it
	enable_hack_trigger_file "light"
	check_is_touch_device && enable_fontconfig_shard "95-force-light"
	kh_msg "Use the custom Light FreeType renderer" I a
}

## Switch to the AutoHint FT lib
ft_autohint()
{
	# Keep things tidy, to avoid getting bit on the ass by the priority rules
	for ft_lib in bci light spr ; do
		[ -f "${KH_HACK_BASEDIR}/${ft_lib}" ] && disable_hack_trigger_file "${ft_lib}"
		check_is_touch_device && disable_fontconfig_shard "95-force-${ft_lib}"
	done
	# Then enable it
	enable_hack_trigger_file "autohint"
	check_is_touch_device && enable_fontconfig_shard "95-force-autohint"
	kh_msg "Use the custom AutoHint FreeType renderer" I a
}

## Switch to the SPR FT lib
ft_spr()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# Keep things tidy, to avoid getting bit on the ass by the priority rules
	for ft_lib in bci light autohint ; do
		[ -f "${KH_HACK_BASEDIR}/${ft_lib}" ] && disable_hack_trigger_file "${ft_lib}"
		check_is_touch_device && disable_fontconfig_shard "95-force-${ft_lib}"
	done
	# Then enable it
	enable_hack_trigger_file "spr"
	check_is_touch_device && enable_fontconfig_shard "95-force-spr"
	kh_msg "Use the custom SPR FreeType renderer" I a
}

## Enable nobrowser mode
enable_nobrowser()
{
	# K3 & K4 only!
	if ! check_is_k3_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	enable_hack_trigger_file "nobrowser"
	kh_msg "Don't handle KF8 & browser fonts" I a "Disable KF8 & browser support"
}

## Disable nobrowser mode
disable_nobrowser()
{
	# K3 & K4 only!
	if ! check_is_k3_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	disable_hack_trigger_file "nobrowser"
	kh_msg "Handle KF8 & browser fonts" I a "Enable KF8 & browser support"
}

## Disable nocondensed mode
disable_nocondensed()
{
	# K3 & K4 only!
	if ! check_is_k3_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	disable_hack_trigger_file "nocondensed"
	kh_msg "Keep the condensed font family option" I a
}

## Enable nocondensed mode
enable_nocondensed()
{
	# K3 & K4 only!
	if ! check_is_k3_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	enable_hack_trigger_file "nocondensed"
	kh_msg "Replace the condensed font family by Caecilia" I a
}

## Disable hide-cjk
show_cjk()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	disable_fontconfig_shard "99-hide-cjk"
	kh_msg "Show CJK Fonts in the aA menu" I a
}

## Enable hide-cjk
hide_cjk()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	enable_fontconfig_shard "99-hide-cjk"
	kh_msg "Hide CJK Fonts from the aA menu" I a
}

## Backup the default fonts
fonts_backup()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# Make sure the target directory exists
	mkdir -p ${LINKFONTS_BASEDIR}/backups

	# Copy!
	cp -f /usr/java/lib/fonts/* ${LINKFONTS_BASEDIR}/backups/

	kh_msg "Default fonts have been backed up" I v
}

## Rebuild fontconfig cache for custom fonts, then restart framework
rebuild_fc_cache()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# Sleep a while to let KUAL die
	kh_msg "Hush, little baby . . ." I a
	sleep 5

	# Update cache!
	${FONTCONFIG_HELPER}

	# Sync, just to be safe
	sync

	# Restart!
	kh_msg "Framework restart . . ." I v
	# Show the splash screen...
	. /etc/upstart/splash
	splash_cleanup
	splash_init || kh_msg "cannot init splash module" E a
	splash_progress 0

	restart webreader
	[ -f "/etc/upstart/kfxreader.conf" ] && restart kfxreader

	# Survive the mass killing so we can properly update the progress bar...
	trap "" SIGTERM

	restart framework

	# And, apparently, we have to do everything ourselves...
	for splash_bar in $(seq 1 25) ; do
		splash_progress $((splash_bar * 4))
		sleep 1
	done
}

## Clear the fontconfig cache, then rebuild it.
clear_fc_cache()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# This is mostly because I sometimes end up with -arm.cache-4 files besides the usual -le32d8.cache-3 when updating the cache...
	for file in /var/cache/fontconfig/* ; do
		[ -f "${file}" ] && rm -f "${file}"
	done

	# Then rebuild it
	kh_msg "updating fontconfig cache . . ." I v
	fc-cache -r
	# Provide some feedback in case it went kablooey...
	if [ $? -eq 0 ] ; then
		kh_msg "fontconfig cache updated successfully :)" I v
	else
		kh_msg "failed to update fontconfig cache :(" W v
		# And leave the user some time to read it...
		sleep 5
	fi

	# Ask the user to restart the device (even if the update failed: it might have only failed halfway trhough, so it's still applicable).
	kh_msg "fontconfig cache updated, you will need to restart your device at one point" I v "done. restart framework/X, then the device"
}

## Move from the lab126 /mnt/us/fonts directory to our own one, to avoid suprises after full official updates...
migrate_fonts_dir()
{
	# K5 & PW only!
	if ! check_is_touch_device ; then
		kh_msg "This is not supported on your device" W v
		return 1
	fi

	# Do we have a legacy font directory?
	if [ ! -d "/mnt/us/fonts" ] ; then
		kh_msg "No legacy fonts directory :)" W v
		return 1
	fi

	# Do eeet!
	mkdir -p /mnt/us/linkfonts/fonts
	mv -f /mnt/us/fonts/* /mnt/us/linkfonts/fonts/
	rm -rf /mnt/us/fonts

	# Be verbose!
	kh_msg "Migration successful. Rebuilding fc cache . . ." I v
	sleep 1

	# And then update fontconfig cache...
	rebuild_fc_cache
}

## Print the version of the hack currently installed
show_version()
{
	if [ -f "${KH_HACK_BASEDIR}/etc/VERSION" ] ; then
		kh_msg "$(cat ${KH_HACK_BASEDIR}/etc/VERSION)" I v
	else
		kh_msg "No version info file" W v
	fi
}

## Main
case "${1}" in
	"enable_auto" )
		${1}
	;;
	"disable_auto" )
		${1}
	;;
	"enable_verbose" )
		${1}
	;;
	"disable_verbose" )
		${1}
	;;
	"enable_uninstall" )
		${1}
	;;
	"disable_uninstall" )
		${1}
	;;
	"enable_autoreboot" )
		${1}
	;;
	"disable_autoreboot" )
		${1}
	;;
	"trigger_autoreboot" )
		${1}
	;;
	"abort_autoreboot" )
		${1}
	;;
	"framework_restart" )
		${1}
	;;
	"webreader_restart" )
		${1}
	;;
	"xorg_restart" )
		${1}
	;;
	"ft_amazon" )
		${1}
	;;
	"ft_bci" )
		${1}
	;;
	"ft_light" )
		${1}
	;;
	"ft_autohint" )
		${1}
	;;
	"ft_spr" )
		${1}
	;;
	"enable_nobrowser" )
		${1}
	;;
	"disable_nobrowser" )
		${1}
	;;
	"disable_nocondensed" )
		${1}
	;;
	"enable_nocondensed" )
		${1}
	;;
	"show_cjk" )
		${1}
	;;
	"hide_cjk" )
		${1}
	;;
	"fonts_backup" )
		${1}
	;;
	"rebuild_fc_cache" )
		${1}
	;;
	"reset_fontconfig" )
		${1}
	;;
	"clear_fc_cache" )
		${1}
	;;
	"migrate_fonts_dir" )
		${1}
	;;
	"show_version" )
		${1}
	;;
	* )
		kh_msg "invalid action (${1})" W v "invalid action"
	;;
esac
